import os
import signal
from multiprocessing import Process, JoinableQueue

from PySide6 import QtCore, QtWidgets
from eipiphany_core.framework.base.source import Source
from eipiphany_core.message.exchange import Exchange


class FileSource(Source):

  DONE_FILE_PATH = 'DoneFilePath'

  def __init__(self, configuration):
    assert configuration.directory, 'directory parameter is required'
    self.__queue = JoinableQueue()
    self.__configuration = configuration
    self.__source_wrapper = None

  def set_source_wrapper(self, source_wrapper):
    self.__source_wrapper = source_wrapper

  def start(self):
    process = []
    supplier = Process(target=self.__source_wrapper.wait_for_events)
    supplier.daemon = True
    supplier.start()
    process.append(supplier)
    consumer = Process(target=self.run_app)
    consumer.daemon = True
    consumer.start()
    process.append(consumer)
    return process

  def __get_filter(self, ext_list):
    filter = []
    for ext in ext_list:
      filter.append('*.{}'.format(ext))
    return filter

  def run_app(self):
    app = QtCore.QCoreApplication([])
    signal.signal(signal.SIGINT, signal.SIG_DFL)
    self.watcher = QtWidgets.QFileSystemModel()
    self.watcher.setNameFilterDisables(False)
    if self.__configuration.done_file_ext:
      if self.__configuration.include_ext:
        exts = []
        for ext in self.__configuration.include_ext:
          exts.append('{}.{}'.format(ext, self.__configuration.done_file_ext))
        self.watcher.setNameFilters(self.__get_filter(exts))
      else:
        self.watcher.setNameFilters(self.__get_filter([self.__configuration.include_ext]))
    else:
      if self.__configuration.include_ext:
        self.watcher.setNameFilters(self.__get_filter(self.__configuration.include_ext))
    self.watcher.setRootPath(self.__configuration.directory)
    self.watcher.rowsInserted.connect(self.__rows_inserted)
    app.exec()

  def __rows_inserted(self, parent, first, last):
    for i in range(first, last + 1):
      child = self.watcher.index(i, 0, parent=parent)
      is_dir = self.watcher.isDir(child)
      if is_dir:
        if self.__configuration.recursive:
          self.watcher.fetchMore(child)
      else:
        path = self.watcher.filePath(child)
        if self.__configuration.done_file_ext:
          actual_path = path[0:( -1 * (len(self.__configuration.done_file_ext) + 1))]
          if not os.path.isfile(actual_path):
            # todo move to error location?
            os.remove(path)
            raise Exception("Done file found, but actual file does not exist: {}".format(actual_path))
          self.__queue.put(Exchange(actual_path).set_header(FileSource.DONE_FILE_PATH, path))
        else:
          self.__queue.put(Exchange(path))

  def wait_for_event(self):
    return self.__queue.get()

  def event_success(self, exchange):
    self.__queue.task_done()
    if exchange.get_header(FileSource.DONE_FILE_PATH):
      os.remove(exchange.get_header(FileSource.DONE_FILE_PATH))
    if self.__configuration.delete:
      os.remove(exchange.body)

  def event_failure(self, err, exchange):
    self.__queue.task_done()
    # todo move this file?
    if exchange.get_header(FileSource.DONE_FILE_PATH):
      os.remove(exchange.get_header(FileSource.DONE_FILE_PATH))
    # todo handle moving on error




from eipiphany_core.framework.base.endpoint import Endpoint

from .file_configuration import FileConfiguration
from .file_source import FileSource


class FileEndpoint(Endpoint):

  def __init__(self, primary_id, source_configuration = None):
    super().__init__(primary_id)
    self.__default_config = FileConfiguration()
    self.__source_configuration = source_configuration
    if not self.__source_configuration:
      self.__source_configuration = self.__default_config
    self.__source = FileSource(self.__source_configuration)

  def get_prefix(self):
    return 'file'

  def process(self, exchange, configuration):
    raise Exception("Not implemented yet")

  def get_source(self):
    return self.__source



class FileConfiguration(object):
  def __init__(self):
    self.__directory = None
    self.__recursive = False
    self.__include_ext = []
    self.__done_file_ext = None
    self.__delete = False
    # self.__noop = True

  @property
  def directory(self):
    return self.__directory

  @directory.setter
  def directory(self, value):
    self.set_directory(value)

  def set_directory(self, value):
    self.__directory = value
    return self

  @property
  def recursive(self):
    return self.__recursive

  @recursive.setter
  def recursive(self, value):
    self.set_recursive(value)

  def set_recursive(self, value):
    self.__recursive = value
    return self

  @property
  def include_ext(self):
    return self.__include_ext

  @include_ext.setter
  def include_ext(self, value):
    self.set_include_ext(value)

  def set_include_ext(self, value):
    if value:
      self.__include_ext = value
    else:
      self.__include_ext = []
    return self

  @property
  def done_file_ext(self):
    return self.__done_file_ext

  @done_file_ext.setter
  def done_file_ext(self, value):
    self.set_done_file_ext(value)

  def set_done_file_ext(self, value):
    self.__done_file_ext = value
    return self

  # @property
  # def noop(self):
  #   return self.__include_ext
  #
  # @noop.setter
  # def noop(self, value):
  #   self.__noop = value
  #   return self
  #
  # def set_noop(self, value):
  #   self.__noop = value
  #   return self

  @property
  def delete(self):
    return self.__delete

  @delete.setter
  def delete(self, value):
    self.set_directory(value)

  def set_delete(self, value):
    self.__delete = value
    return self

  # todo
  def __str__(self):
    return 'FileSourceConfiguration:' \
           '{{' \
           'directory={},' \
           'recursive={}' \
           '}}'\
      .format(
      self.__directory,
      self.__recursive
    )